///////////////////////////////////////////////////////////////////////////////
//                                                                           //
//                     Lookup Tables, maths and 3D extracts                  //
//                             John Biddiscombe                             //
//                           J.Biddiscombe@rl.ac.uk                          //
//                                  Freeware                                 //
//                                                                           //
///////////////////////////////////////////////////////////////////////////////
unit Math_utils;

interface

uses
  { Borland }
  Math;

type
  coord_3D                = array[0..3] of double;
  matrix_3D               = array[0..3] of coord_3D;

const
  tablesize               = 16383;
  tablesizep1             = tablesize+1;
  tablesized2             = tablesizep1 div 2;

type
  table = array[0..tablesize] of double;

var
  cosine   : table;
  sine     : table;
  sigmoids : table;
  theta_d  : array[1..10] of integer;
  { geometry }
  rotyxz0_0,
  rotyxz0_1,
  rotyxz0_2,
  rotyxz1_0,
  rotyxz1_1,
  rotyxz1_2,
  rotyxz2_1,
  rotyxz2_2,
  rotyxz2_3  : double;


procedure setup_tables;
procedure yxz_rotation(x,y,z:integer); { unrolled matrix multiply }
function  powerxy(x,y:double):double;
function  ranbit : integer;
function  gauss_mean_sigma(mu,sigma:double): double;

implementation

procedure setup_tables;
var lp1,lp2 : integer;
begin
  for lp1:=0 to tablesize do begin
    cosine[lp1]   := cos(2*pi*lp1/tablesizep1);
    sine[lp1]     := sin(2*pi*lp1/tablesizep1);
    sigmoids[lp1] := 0.5*(1+tanh(6.0*(lp1-tablesized2)/(1.0*tablesizep1)));
  end;
  for lp1:=1 to 10 do theta_d[lp1] := tablesizep1 div lp1;
end;

{ ------------------------------------------------------------------------ }
procedure yxz_rotation(x,y,z:integer); { unrolled matrix multiply }
var t1,t2,t3,t4,t5,t6,m1,m2,m3 : double;
begin
  t1:=cosine[x];
  t2:=cosine[y];
  t3:=cosine[z];
  t4:=  sine[x];
  t5:=  sine[y];
  t6:=  sine[z];
  m1 := (t2*t3);
  m2 := (t4*t5);
  m3 := (t2*t6);
  rotyxz0_0 := m1         - ((m2*t6));
  rotyxz0_1 :=-((t1*t6));
  rotyxz0_2 := ((t3*t5))  + ((m3*t4));
  rotyxz1_0 := m3         + ((t3*m2));
  rotyxz1_1 := ((t1*t3));
  rotyxz1_2 := ((t5*t6))  - ((m1*t4));
  rotyxz2_1 :=-((t1*t5));
  rotyxz2_2 := t4;
  rotyxz2_3 := ((t1*t2));
end;
{ ------------------------------------------------------------------------ }
function powerxy(x,y:double):double;
begin
  if x>0 then result:=exp(ln(x)*y)
  else if x=0 then result:=0
  else begin
    if (round(y) and 1)=1 then result:=-exp(ln(-x)*y)
    else                       result:= exp(ln(-x)*y);
  end;
end;
{ ------------------------------------------------------------------------ }
var
   newseed           : longint;
   gliset            : longint;
   glgset            : double;
   idum              : longint;
   glix1,glix2,glix3 : longint;
   glr               : ARRAY [1..97] OF double;
{ ------------------------------------------------------------------------ }
function ranbit : integer;
begin if (random(1024) and 8)=8 then ranbit:=1 else ranbit:=-1; end;
{ ------------------------------------------------------------------------ }
function ran1 : double;
CONST
   m1=259200;
   ia1=7141;
   ic1=54773;
   rm1=3.8580247e-6;   (* 1.0/m1 *)
   m2=134456;
   ia2=8121;
   ic2=28411;
   rm2=7.4373773e-6;   (* 1.0/m2 *)
   m3=243000;
   ia3=4561;
   ic3=51349;
VAR
   j: longint;
BEGIN
   IF  (idum < 0)  THEN BEGIN
      glix1 := (ic1-idum) MOD m1;
      glix1 := (ia1*glix1+ic1) MOD m1;
      glix2 := glix1 MOD m2;
      glix1 := (ia1*glix1+ic1) MOD m1;
      glix3 := glix1 MOD m3;
      FOR j := 1 TO 97 DO BEGIN
         glix1 := (ia1*glix1+ic1) MOD m1;
         glix2 := (ia2*glix2+ic2) MOD m2;
         glr[j] := (glix1+glix2*rm2)*rm1
      END;
      idum := 1
   END;
   glix1 := (ia1*glix1+ic1) MOD m1;
   glix2 := (ia2*glix2+ic2) MOD m2;
   glix3 := (ia3*glix3+ic3) MOD m3;
   j := 1 + (97*glix3) DIV m3;
   IF ((j > 97) OR (j < 1)) THEN BEGIN
      writeln('pause in routine RAN1'); readln
   END;
   ran1 := glr[j];
   glr[j] := (glix1+glix2*rm2)*rm1;
END;
{ ------------------------------------------------------------------------ }
function gasdev : double;
VAR
   fac,r,v1,v2: double;
BEGIN
   IF  (gliset = 0)  THEN BEGIN
      REPEAT
         v1 := 2.0*ran1-1.0;
         v2 := 2.0*ran1-1.0;
         r := sqr(v1)+sqr(v2);
      UNTIL (r < 1.0);
      fac := sqrt(-2.0*ln(r)/r);
      glgset := v1*fac;
      gasdev := v2*fac;
      gliset := 1
   END ELSE BEGIN
      gasdev := glgset;
      gliset := 0
   END
END;
{ ------------------------------------------------------------------------ }
function gauss_mean_sigma(mu,sigma:double): double;
begin
   gauss_mean_sigma:=mu+(sigma*gasdev);
end;
{ ------------------------------------------------------------------------ }

begin
  Randomize;
  gliset  := 0;
  idum    := -random(256);
  newseed := random(65535);
  newseed := 44;
  { use idum =-1 if problems encountered }
end.

